@@ -1,40 +1,17 @@
 import re
-from collections import defaultdict
 
-try:
-    basestring
-except NameError: # python 3
-    basestring = str
+from .types import (convertValueToObjc as convertValueToObjcBase, KeyValueId, ConstGenerator,
+    NLSTR, Binding, generateDictionary)
+from .property import Property
+
+globalLocalizationTable = None
+globalRunMode = False
 
 def upFirstLetter(s):
     return s[0].upper() + s[1:]
 
-def stringArray(strings):
-    return "[NSArray arrayWithObjects:%s,nil]" % ','.join(('@"%s"' % s) for s in strings)
-
-globalLocalizationTable = None
-
 def convertValueToObjc(value):
-    if value is None:
-        return 'nil'
-    elif isinstance(value, GeneratedItem):
-        return value.varname
-    elif isinstance(value, KeyValueId):
-        return value._objcAccessor()
-    elif isinstance(value, Literal):
-        return value.value
-    elif isinstance(value, basestring):
-        value = value.replace('\n', '\\n')
-        result = '@"%s"' % value
-        if value and globalLocalizationTable:
-            result = 'NSLocalizedStringFromTable(%s, @"%s", @"")' % (result, globalLocalizationTable)
-        return result
-    elif isinstance(value, bool):
-        return 'YES' if value else 'NO'
-    elif isinstance(value, (int, float)):
-        return str(value)
-    else:
-        raise TypeError("Can't figure out the property's type")
+    return convertValueToObjcBase(value, localizationTable=globalLocalizationTable)
 
 class CodeTemplate(object):
     def __init__(self, template):
@@ -69,120 +46,17 @@ class CodeTemplate(object):
             placeholders = re.findall(r"\$\w+?\$", result)
         return result
 
-class KeyValueId(object):
-    # When we set an KeyValueId attribute in our source file, there no convenient way of saying,
-    # at the codegen phase "this is exactly when this value was set, so I'll insert code to assign
-    # this value here." What we can do, however, is having a dictionary of all keys a certain value
-    # was assigned to and when we create the code for that value, we insert assignments right after.
-    VALUE2KEYS = defaultdict(set)
-    def __init__(self, parent, name, fakeParent=False):
-        self._parent = parent
-        self._name = name
-        # set fakeParent to True when you want to ignore this KeyValueId in accessors. You can use
-        # this for stuff like "const.NSOnState" where we want the accessor to be "NSOnState", not
-        # [const NSOnState];
-        self._fakeParent = fakeParent
-        self._children = {}
-    
-    def __repr__(self):
-        return '<KeyValueId %s>' % self._objcAccessor()
-    
-    def __getattr__(self, name):
-        if name.startswith('_'):
-            return object.__getattribute__(self, name)
-        if name in self._children:
-            result = self._children[name]
-        else:
-            result = KeyValueId(self, name)
-            self._children[name] = result
-        return result
-    
-    def __setattr__(self, name, value):
-        if name.startswith('_'):
-            object.__setattr__(self, name, value)
-            return
-        key = getattr(self, name)
-        KeyValueId.VALUE2KEYS[value].add(key)
-    
-    # the methods below aren't actually private, it's just that we prepend them with underscores to
-    # avoid name clashes.
-    def _objcAccessor(self):
-        if self._parent and not self._parent._fakeParent:
-            if self._parent._name == 'nil':
-                return 'nil'
-            else:
-                return '[%s %s]' % (self._parent._objcAccessor(), self._name)
-        else:
-            return self._name
-    
-    def _callMethod(self, methodname, argument=None, endline=True):
-        # For now, this method only supports call to methods of zero or one argument.
-        if argument is None:
-            result = getattr(self, methodname)._objcAccessor()
-        else:
-            result = '[%s %s:%s]' % (self._objcAccessor(), methodname, convertValueToObjc(argument))
-        if endline:
-            result += ';\n'
-        return result
-    
-    def _clear(self):
-        for child in self._children.values():
-            child._clear()
-        self._children.clear()
-        for keys in KeyValueId.VALUE2KEYS.values():
-            keys.discard(self)
-    
-
 owner = KeyValueId(None, 'owner')
 NSApp = KeyValueId(None, 'NSApp')
-const = KeyValueId(None, 'const', fakeParent=True)
-
-class Action(object):
-    def __init__(self, target, selector):
-        self.target = target
-        self.selector = selector
-    
-    def generate(self, sender):
-        tmpl = CodeTemplate("""[$sender$ setTarget:$target$];
-        [$sender$ setAction:$selector$];
-        """)
-        tmpl.sender = sender
-        tmpl.selector = "@selector(%s)" % self.selector
-        if self.target:
-            tmpl.target = self.target._objcAccessor()
-        else:
-            tmpl.target = 'nil'
-        return tmpl.render()
-    
-
-class KeyShortcut(object):
-    def __init__(self, shortcutStr):
-        self.shortcutStr = shortcutStr
-        elements = set(shortcutStr.lower().split('+'))
-        flags = []
-        availableFlags = [
-            ('cmd', 'NSCommandKeyMask'),
-            ('ctrl', 'NSControlKeyMask'),
-            ('alt', 'NSAlternateKeyMask'),
-            ('shift', 'NSShiftKeyMask'),
-        ]
-        for ident, flag in availableFlags:
-            if ident in elements:
-                elements.remove(ident)
-                flags.append(flag)
-        self.flags = '|'.join(flags)
-        assert len(elements) == 1
-        self.key = list(elements)[0]
-        
-
-# Use this in properties when you need it to be generated as-is, and not wrapped as a normal string
-class Literal(object):
-    def __init__(self, value):
-        self.value = value
-    
+const = ConstGenerator()
+defaults = KeyValueId(None, 'NSUserDefaultsController').sharedUserDefaultsController
 
 class GeneratedItem(object):
     OBJC_CLASS = 'NSObject'
+    # This is a shorthand for setting the self.properties dictionary with the value of the prop in
+    # generateInit(). This list contains either Property instances or, to avoid unnecessary
+    # verbosity, a string with the property name, which is the equivalent of Property(name).
+    PROPERTIES = []
     
     def __init__(self):
         self.creationOrder = globalGenerationCounter.creationToken()
@@ -191,6 +65,33 @@ class GeneratedItem(object):
         # properties to be set at generation time. For example, if "editable" is set to False,
         # a "[$varname$ setEditable:NO];" statement will be generated.
         self.properties = {}
+        self._bindings = []
+    
+    #--- Private
+    def _generateProperties(self, properties=None):
+        result = ''
+        if properties is None:
+            properties = self.properties
+            for prop in self.PROPERTIES:
+                if not isinstance(prop, Property):
+                    assert isinstance(prop, str)
+                    prop = Property(prop)
+                prop.setOnTarget(self)
+        for key, value in properties.items():
+            if value is None:
+                continue
+            dot_elements = key.split('.')
+            accessor = self.accessor
+            for dot_element in dot_elements[:-1]:
+                accessor = getattr(accessor, dot_element)
+            if isinstance(value, GeneratedItem) and not value.generated:
+                # Generate an assignment (which is generated by the "value" part of the assignment)
+                # so that we set that value after our target item was generated
+                setattr(accessor, dot_elements[-1], value)
+            else:
+                methname = 'set' + upFirstLetter(dot_elements[-1])
+                result += accessor._callMethod(methname, value)
+        return result
     
     #--- Virtual
     def generateInit(self):
@@ -215,6 +116,16 @@ class GeneratedItem(object):
     def generated(self):
         return globalGenerationCounter.isGenerated(self)
     
+    def bind(self, name, target, keyPath, valueTransformer=None):
+        options = {}
+        if valueTransformer:
+            options[const.NSValueTransformerNameBindingOption] = NLSTR(valueTransformer)
+        binding = Binding(NLSTR(name), target, NLSTR(keyPath), options)
+        self._bindings.append(binding)
+    
+    def objcValue(self):
+        return self.varname
+    
     def generateAssignments(self):
         if self not in KeyValueId.VALUE2KEYS:
             return ""
@@ -225,6 +136,20 @@ class GeneratedItem(object):
             assignments.append(assignment)
         return '\n'.join(assignments)
     
+    def generateBindings(self):
+        bindings = []
+        for binding in self._bindings:
+            method = '[{} bind:{} toObject:{} withKeyPath:{} options:{}];'
+            if binding.options:
+                options = generateDictionary(binding.options)
+            else:
+                options = 'nil'
+            name = convertValueToObjc(binding.name)
+            target = convertValueToObjc(binding.target)
+            keyPath = convertValueToObjc(binding.keyPath)
+            bindings.append(method.format(self.varname, name, target, keyPath, options))
+        return '\n'.join(bindings)
+    
     def generateFinalize(self):
         # Called after everything has been generated.
         pass
@@ -235,24 +160,13 @@ class GeneratedItem(object):
             if isinstance(dependency, GeneratedItem) and not dependency.generated:
                 result += dependency.generate()
         inittmpl = self.generateInit(*args, **kwargs)
-        setprop = ''
-        for key, value in self.properties.items():
-            if value is None:
-                continue
-            dot_elements = key.split('.')
-            accessor = self.accessor
-            for dot_element in dot_elements[:-1]:
-                accessor = getattr(accessor, dot_element)
-            if isinstance(value, GeneratedItem) and not value.generated:
-                # Generate an assignment (which is generated by the "value" part of the assignment)
-                # so that we set that value after our target item was generated
-                setattr(accessor, dot_elements[-1], value)
-            else:
-                methname = 'set' + upFirstLetter(dot_elements[-1])
-                setprop += accessor._callMethod(methname, value)
-        inittmpl.setprop = setprop
+        inittmpl.setprop = self._generateProperties()
         result += inittmpl.render()
         result += self.generateAssignments()
+        if not globalRunMode:
+            # We don't generate bindings in "run" mode because bindings can generate crashes if they
+            # aren't actually connected to something.
+            result += self.generateBindings()
         globalGenerationCounter.addGenerated(self)
         return result
     
