@@ -2,7 +2,8 @@ from __future__ import division
 
 from collections import namedtuple, defaultdict
 
-from .base import GeneratedItem, Literal
+from .base import GeneratedItem
+from .types import Literal
 
 class Pack(object):
     # Corners
@@ -39,11 +40,52 @@ class Pack(object):
             return 'above'
         elif side == Pack.Below:
             return 'below'
+    
+    @staticmethod
+    def isSide(sideOrCorner):
+        return sideOrCorner in {Pack.Left, Pack.Right, Pack.Above, Pack.Below, Pack.Middle}
+    
+    @staticmethod
+    def isCorner(sideOrCorner):
+        return sideOrCorner in {Pack.UpperLeft, Pack.UpperRight, Pack.LowerLeft, Pack.LowerRight}
+    
+    @staticmethod
+    def sidesInCorner(corner):
+        assert Pack.isCorner(corner)
+        if corner == Pack.UpperLeft:
+            return {Pack.Left, Pack.Above}
+        elif corner == Pack.UpperRight:
+            return {Pack.Right, Pack.Above}
+        elif corner == Pack.LowerRight:
+            return {Pack.Right, Pack.Below}
+        elif corner == Pack.LowerLeft:
+            return {Pack.Left, Pack.Below}
 
 Anchor = namedtuple('Anchor', 'corner growX growY')
 
+class Size(object):
+    def __init__(self, width, height):
+        self.width = width
+        self.height = height
+    
+    def objcValue(self):
+        return 'NSMakeSize(%d, %d)' % (self.width, self.height)
+    
+
+class Rect(object):
+    def __init__(self, x, y, width, height):
+        self.x = x
+        self.y = y
+        self.width = width
+        self.height = height
+    
+    def objcValue(self):
+        return 'NSMakeRect(%d, %d, %d, %d)' % (self.x, self.y, self.width, self.height)
+    
+
 class View(GeneratedItem):
     OBJC_CLASS = 'NSView'
+    PROPERTIES = GeneratedItem.PROPERTIES + ['menu']
     
     INNER_MARGIN_LEFT = 20
     INNER_MARGIN_RIGHT = 20
@@ -77,7 +119,11 @@ class View(GeneratedItem):
         self.layoutDeltaY = 0
         self.layoutDeltaW = 0
         self.layoutDeltaH = 0
-        
+    
+    def _updatePos(self):
+        # This is called after the view had its position changed by a layout method. Here, we do
+        # nothing, but the Layout subclass does.
+        pass
     
     #--- Layout
     def outerMargin(self, other, side):
@@ -91,27 +137,67 @@ class View(GeneratedItem):
     def innerMargin(self, side):
         return getattr(self, 'INNER_MARGIN_' + Pack.side2str(side).upper())
     
-    def packToCorner(self, corner):
+    def innerMarginDelta(self, side):
+        # This is called in pair with innerMargin to the child view for which we need to know where
+        # to place inside a view. Most of the time, we just user innerMargin, but some view, like
+        # the TabView, are placed closer to the border.
+        return 0
+    
+    # The x, y, width and height pos represent the **layout* rect. To have a frame rect, we need
+    # to apply deltas.
+    def frameRect(self):
+        x, y, w, h = self.x, self.y, self.width, self.height
+        x += self.layoutDeltaX
+        y += self.layoutDeltaY
+        w += self.layoutDeltaW
+        h += self.layoutDeltaH
+        return x, y, w, h
+    
+    # Return True if the view can't have its width or height modified by layout methods.
+    def hasFixedWidth(self):
+        return False
+    
+    def hasFixedHeight(self):
+        return False
+    
+    def isOrHas(self, viewtype, side):
+        # Returns whether self is of type `viewtype` or if it contains a view, touching `side`,
+        # that is of that type (Overridden by layouts)
+        return isinstance(self, viewtype)
+    
+    def packToCorner(self, corner, margin=None):
+        def getmargin(side):
+            if margin is not None:
+                return margin
+            else:
+                result = self.parent.innerMargin(side)
+                result += self.innerMarginDelta(side)
+                return result
+        
         assert self.parent is not None
         px, py, pw, ph = self.parent.rect
         x, y, w, h = self.rect
         if corner in (Pack.LowerLeft, Pack.UpperLeft):
-            x = self.parent.innerMargin(Pack.Left)
+            x = getmargin(Pack.Left)
         else:            
             x = pw - self.parent.innerMargin(Pack.Right) - w
         if corner in (Pack.LowerLeft, Pack.LowerRight):
-            y = self.parent.innerMargin(Pack.Below)
+            y = getmargin(Pack.Below)
         else:            
-            y = ph - self.parent.innerMargin(Pack.Above) - h
+            y = ph - getmargin(Pack.Above) - h
         self.x, self.y = x, y
+        self._updatePos()
     
-    def packRelativeTo(self, other, side, align=None):
+    def packRelativeTo(self, other, side, align=None, margin=None):
         assert other.parent is self.parent
         ox, oy, ow, oh = other.rect
         x, y, w, h = self.rect
-        outerMargin1 = self.outerMargin(other, side)
-        outerMargin2 = other.outerMargin(self, Pack.oppositeSide(side))
-        outerMargin = max(outerMargin1, outerMargin2)
+        if margin is not None:
+            outerMargin = margin
+        else:
+            outerMargin1 = self.outerMargin(other, side)
+            outerMargin2 = other.outerMargin(self, Pack.oppositeSide(side))
+            outerMargin = max(outerMargin1, outerMargin2)
         
         if align is None:
             align = Pack.Left if side in (Pack.Above, Pack.Below) else Pack.Middle
@@ -141,25 +227,42 @@ class View(GeneratedItem):
         self.x, self.y = x, y
         self.neighbors[Pack.oppositeSide(side)].add(other)
         other.neighbors[side].add(self)
+        self._updatePos()
     
     def setAnchor(self, corner, growX=False, growY=False):
         self.anchor = Anchor(corner, growX, growY)
     
-    def fill(self, side):
+    def fill(self, side, margin=None, goal=None):
+        def getmargin(side):
+            if margin is not None:
+                return margin
+            else:
+                return self.parent.innerMargin(side)
+        
+        if Pack.isCorner(side):
+            for side in Pack.sidesInCorner(side):
+                self.fill(side, margin=margin)
+            return
+        if side in {Pack.Left, Pack.Right} and self.hasFixedWidth():
+            return
+        if side in {Pack.Above, Pack.Below} and self.hasFixedHeight():
+            return        
         assert self.parent is not None
         px, py, pw, ph = self.parent.rect
         x, y, w, h = self.rect
         neighbors = self.neighbors[side]
         if side == Pack.Right:
             nx = max([(n.x + n.width) for n in neighbors] + [x+w])
-            goal = pw - self.parent.innerMargin(Pack.Right)
+            if goal is None:
+                goal = pw - getmargin(Pack.Right)
             growby = goal - nx
             w += growby
             for n in neighbors:
                 n.x += growby
         elif side == Pack.Left:
             nx = min([n.x for n in neighbors] + [x])
-            goal = self.parent.innerMargin(Pack.Left)
+            if goal is None:
+                goal = getmargin(Pack.Left)
             growby = nx - goal
             w += growby
             x -= growby
@@ -167,27 +270,35 @@ class View(GeneratedItem):
                 n.x -= growby
         elif side == Pack.Below:
             ny = min([n.y for n in neighbors] + [y])
-            goal = self.parent.innerMargin(Pack.Below)
+            if goal is None:
+                goal = getmargin(Pack.Below)
             growby = ny - goal
             h += growby
             y -= growby
             for n in neighbors:
                 n.y -= growby
+        elif side == Pack.Above:
+            ny = max([n.y + n.height for n in neighbors] + [y+h])
+            if goal is None:
+                goal = ph - getmargin(Pack.Above)
+            growby = goal - ny
+            h += growby
+            for n in neighbors:
+                n.y += growby
         else:
-            raise Exception("Above fill not supported yet")
+            raise ValueError("Wrong side argument")
         self.x, self.y, self.width, self.height = x, y, w, h
+        self._updatePos()
+        for n in neighbors:
+            n._updatePos()
     
     #--- Generate
     def generateInit(self):
         tmpl = GeneratedItem.generateInit(self)
         tmpl.setup = "$viewsetup$\n$addtoparent$\n"
         tmpl.initmethod = "initWithFrame:$rect$"
-        x, y, w, h = self.x, self.y, self.width, self.height
-        x += self.layoutDeltaX
-        y += self.layoutDeltaY
-        w += self.layoutDeltaW
-        h += self.layoutDeltaH
-        tmpl.rect = "NSMakeRect(%d, %d, %d, %d)" % (x, y, w, h)
+        x, y, w, h = self.frameRect()
+        tmpl.rect = Rect(x, y, w, h).objcValue()
         if self.anchor.growX and self.anchor.growY:
             resizeMask = 'NSViewWidthSizable|NSViewHeightSizable'
         elif self.anchor.growX:
